package CTL;

import NCLib.*;

import java.io.*;
import java.util.*;

/** Toolbox for managing CTL4j applications and components */
public class Admin extends ToolBase
{
	private final static boolean debug = false;
	
	private static String read1 ()
	{
		Reader read = new InputStreamReader(System.in);
		BufferedReader read2 = new BufferedReader(read);
		try
		{
			return read2.readLine();
		}
		catch (Exception e)
		{
			return null;
		}
	}
	
	private static void help ()
	{
		System.out.println("\nCTL4j command line interface\n");
		System.out.println("Usage: CTL.Admin [command] [options]\n");
		System.out.println("Available commands:");
		System.out.println("quickstart	Create a new CTL4j project");
		System.exit(1);
	}

	private static void quickstart ()
	{
		String dir = new Admin().whereAmI(Type.FILE);
		String name = "foobar";
		String comp = "moo";
		String dest = "/tmp";
		if (!debug)
		{
			System.out.print("Name of your project: ");
			name = read1();
			System.out.print("Name of your component: ");
			comp = read1();
			dest = System.getenv("PWD");
		}
		dest += "/" + name;
		
		Files.mkdirOrDie(dest);
		Files.mkdirOrDie(dest+"/src");
		Files.mkdirOrDie(dest+"/src/Impl");
		Files.mkdirOrDie(dest+"/src/javaSys");
		
		//Files.symlink(dir+"/src/Example.Server", dest+"/src/Server_");
		Template Server_ = new Template(dir+"/src/Example.Server");
		Server_.set("ctlcp", " Added by CTL.Admin\nCP=\"\\$CP:"+dir+"/build\"\n"+
			"CP=\"\\$CP:`find "+dir+" -name '*.jar' -print0|sed 's/jar/jar:/g'`\"\n");
		Server_.write(dest+"/src/Server_");
		Files.changePermissions("775", dest+"/src/Server_");
		
		//Files.writeAll(dest+"/src/.cvsignore", "javaSys\n");
		Files.writeAll(dest+"/locs.txt", "#lib\n"+System.getenv("USER")+"@localhost:"+
			dest+"/src/Server_ tcp\n");
		Files.writeAll(dest+"/src/Impl/"+comp+".java", 
			"package Impl;\n\npublic class "+comp+"\n{\n}\n");
		Files.writeAll(dest+"/src/Server.java",
			Files.readAll(dir+"/data/quickstart/Server.java"));
		
		Template buildXML = new Template(dir+"/data/quickstart/build.xml");
		buildXML.set("ctlloc", dir);
		buildXML.set("comp", comp);
		buildXML.write(dest+"/build.xml");

		Template client = new Template(dir+"/data/quickstart/Client.java");
		client.set("comp", comp);
		client.write(dest+"/src/Client.java");
		
		System.exit(0);
	}
	
	public static void main (String[] args)
	{
		if (debug)
			quickstart();
		if (args.length < 1)
			help();
		if (args[0].equals("quickstart"))
			quickstart();	
		help();
	}
}
