package CTL.Comm;

import CTL.*;
import CTL.Types.*;
import NCLib.*;

import java.io.*;
import java.net.*;

// TODO: error handling

/** Communication over SSH pipes
 * @see TCPCommunicator
 */
public class PipeCommunicator extends Communicator
{	
	public enum Type { CLI, SRV };

	private String inPipe = null, outPipe = null;
	private InputStream in = null;
	private OutputStream out = null;

	public String getRedirect ()
	{
		return " <"+outPipe+" >"+inPipe;
	}

	public PipeCommunicator (Type typ)
	{
		switch (typ)
		{
			case CLI:
				// Using native mkfifo() implementation in libunix.so
				Location loc = new Location("./cpp/libunix/libunix.so lib");
				CTL.Process proc = new CTL.Process(loc);
				UnixCI.use(proc);
				UnixCI tmp = new UnixCI();

				inPipe = tmp.mkfifo();
				outPipe = tmp.mkfifo();

				if ((inPipe == null) || (outPipe == null))
				{
					System.out.println("Could not create pipes.");
					System.exit(1);
				}
				break;
			case SRV:
				in = System.in;
				out = System.out;
				break;
			default:
				System.out.println("PipeCommunicator: Never reached.");
				System.exit(1);
				break;
		}
	}

	public PeerID pid ()
	{
		IPaddr magic = new IPaddr(CTL.Remote.magic, 0);
		return new PeerID(magic, -1);
	}

	public void send (byte[] data, long len, boolean debug) throws IOException
	{
		out.write(data, 0, (int)len);
		out.flush();
		Env.log.msg(Logger.DBG2, "send() "+len+" bytes written.");
	}

	public byte[] recv () throws IOException, CTLException
	{
		while (in.available() == 0);
		return Remote.readMessage(in);
	}
	
	public byte[] recv (long len) throws IOException, CTLException
	{
		while (in.available() == 0);
		byte[] buf = new byte[(int)len];
		int read = in.read(buf);
		assert read == len;
		return buf;
	}

	public void close () throws IOException
	{
		if (in != null)
			in.close();
		if (out != null)
			out.close();
		// TODO: Investigate deleteOnExit()
		if (inPipe != null)
			new File(inPipe).delete();
		if (outPipe != null)
			new File(outPipe).delete();
	}
	
	public void accept () throws IOException
	{
		try
		{
			/* 
			 * Order is important, because open() on a FIFO blocks
			 * until it is opened by one process for reading and
			 * another one for writing (see mkfifo(3)).
			 */
			out = new FileOutputStream(new File(outPipe));
			in = new FileInputStream(new File(inPipe));
		}
		catch (FileNotFoundException e)
		{
			// Never reached
		}
	}

	public void listen () throws IOException { }
	public void connect (PeerID remote) throws UnknownHostException, 
		IOException { }
}
