package CTL.Comm;

import CTL.*;
import CTL.Streams.*;
import CTL.Types.*;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.nio.channels.*;

/** TCP/IP communication handling */
public class TCPCommunicator extends Communicator
{
	/** Local IP */
	private IPaddr host = null;
	/** Local port */
	private int port = 0;
	/** Selector for this communicator */
	@SuppressWarnings("unused")
	private Selector sel = null;
	/** Socket connection to the communication partner */
	private Socket sock = null;
	/** Socket connection to the remote debugger */
	private Socket dbgsock = null;

	// TODO: Implement NIO for accept()
	private ServerSocket ssock = null;
	
	/** Change the contained IP address
	 * @param host Host
	 */
	public void setHost (IPaddr host)
	{
		this.host = host;
	}

	/** Default constructor */
	public TCPCommunicator ()
	{
		try
		{
			sel = Selector.open();
		}
		catch (IOException e)
		{
			RUtil.except(e);
		}
		
		this.host = new IPaddr();
	}

	/** Constructor for known remote host and port */
	public TCPCommunicator (String host, int port, int lport)
	{
		this();
		this.host = determineIP(host, port);
		this.port = lport;
	}

	/** Constructor for known remote port */
	public TCPCommunicator (int port)
	{
		this();
		this.port = port;
	}

	/** Accept connections */
	public void accept () throws IOException
	{
		sock = ssock.accept();
		host = new IPaddr(sock.getLocalAddress());
		
		/*running = true;
		while (running && (sock == null))
		{
			int num;
			Set<SelectionKey> selectedKeys;
			
			try
			{
				num = sel.select();
				selectedKeys = sel.selectedKeys();
			}
			catch (IOException e)
			{
				RUtil.except(e);
				continue;
			}
			
			for (SelectionKey key : selectedKeys)
			{
				if ((key.readyOps() & SelectionKey.OP_ACCEPT) == SelectionKey.OP_ACCEPT)
				{
					try
					{
						ServerSocketChannel ssc = (ServerSocketChannel)key.channel();
						SocketChannel sc = ssc.accept();
						selectedKeys.remove(key);
						// Now the real IP can be determined from the connection's socket.
						sock = sc.socket();
						host = new IPaddr(sock.getLocalAddress());
					}
					catch (IOException e)
					{
						RUtil.except(e);
					}
				}
			}
		}*/
	}	

	/** Listen for new connections */
	public void listen () throws IOException
	{
		ssock = new ServerSocket(port, 25);
		/*ServerSocketChannel ssc = ServerSocketChannel.open();
		ssc.configureBlocking(false);
		ServerSocket ssock = ssc.socket();
		InetSocketAddress address = new InetSocketAddress(port);
		ssock.bind(address);
		SelectionKey key = ssc.register(sel, SelectionKey.OP_ACCEPT);*/
		port = ssock.getLocalPort();
		Env.log.msg(Logger.DBG1, "Listening on "+ssock.getLocalPort()+".");
	}

	/** Retrieve the PeerID
	 * @return PeerID
	 */
	public PeerID pid ()
	{
		Env.log.msg(Logger.DBG2, ""+port);
		return new PeerID(host, port);
	}

	/** Connect to peer 
	 * @param pid PeerID of remote host
	 */
	public void connect (PeerID pid) 
		throws UnknownHostException, IOException
	{
		sock = new Socket(pid.host().toString(), pid.port());
		this.host = new IPaddr(sock.getLocalAddress());
		this.port = sock.getLocalPort();
	}

	/** Initialize connection to debugger
	 * @param dbgHost Location of the debugger
	 */
	private void attachDebugger (PeerID dbgHost) throws UnknownHostException,
		IOException
	{
		if (dbgHost == null)
		{
			Env.log.msg(Logger.ERR, "Can not attach NULL debugger.");
			return;
		}
		String host = dbgHost.host().toString();
		int port = dbgHost.port();
		dbgsock = new Socket(host, port);
		Env.log.msg(Logger.DBG, "Debugger attached from "+host+":"+port+"\n");
	}	

	/** Close the connection */
	public void close () throws IOException
	{
		sock.close();
		if (dbgsock != null)
			dbgsock.close();
	}

	/** Send a specific amount of data
	 * @param data Bytes to send
	 * @param len Number of bytes to send
	 * @param debug Whether or not to send the data to the debugger
	 */
	public void send (byte[] data, long len, boolean debug) throws IOException
	{
		if (debug && (Env.debugger != null))
		{
			// writeCopy()
			sendDebug(data);
			// wait()
			if (Globals.doWait)
			{
				/*System.out.println("Waiting for continuous message...");
				if (Env.debugID != null)
					System.out.println("Debug now: jdb -attach localhost:"+
						Env.debugID);*/

				try
				{
					OIStream in = new OIStream(dbgsock.getInputStream());
					Header head = in.serialRead(Header.class);
				}
				catch (Exception e)
				{
					Env.log.msg(Logger.ERR, "Read error");
					RUtil.except(e);
				}
			}
		}

		// writeOrig()
		OutputStream out = sock.getOutputStream();
		Env.log.msg(Logger.INFO, "Writing "+(data.length-Remote.headerSize)+" bytes.");
		out.write(data);
		out.flush();
	}

	/** Send data to the debugger 
	 * @param data Bytes to send
	 */
	public void sendDebug (byte[] data) throws IOException
	{
		if (dbgsock == null)
			attachDebugger(Env.debugger);
		OutputStream out = dbgsock.getOutputStream();
		out.write(data);
		out.flush();
	}

	/** Receive a specific amount of data
	 * @param len Number of bytes to receive
	 * @return Data
	 */
	public byte[] recv (long len) throws IOException, CTLException
	{
		InputStream inp = sock.getInputStream();
		new OIStream(inp);
		byte[] buf = new byte[(int)len];
		inp.read(buf);
		return buf;
	}

	/** Receive a CTL packet (length is determined from header)
	 * @return Data
	 */
	public byte[] recv () throws IOException, CTLException
	{
		return Remote.readMessage(sock.getInputStream());

		/*try
		{
			int headN = RUtil.objSize(new Header());
			InputStream inp = sock.getInputStream();
			byte[] buf = new byte[headN];
			inp.read(buf);

			if (buf == null)
				Env.log.msg(Logger.ERR, "No data received.");
			
			BuffyIn in = new BuffyIn(buf);
			Header head = in.i.serialRead(Header.class);
			
			if (head == null)
			{
				Env.log.msg(Logger.ERR, "Empty header.");
				return null;
			}
			if (head.getSize() > 64*1024*1024)
			{
				Env.log.msg(Logger.ERR, "Header size larger than 64MB.");
				System.out.println("Possible header corruption detected.");
				System.exit(11);
			}

			byte[] buf2 = new byte[(int)head.getSize()];
			Env.log.msg(Logger.INFO, "Read "+head.getSize()+" bytes.");
			inp.read(buf2);
		
			byte[] buf3 = new byte[(int)head.getSize()+headN];
			for (int i=0;i<headN;i++)
				buf3[i] = buf[i];
			for (int i=0;i<(int)head.getSize();i++)
				buf3[headN+i] = buf2[i];

			Env.log.msg(Logger.DBG2, "recv() Header: "+head);

			return buf3;
		} 
		catch (IllegalAccessException e0) {}
		catch (ClassNotFoundException e1) {}
		catch (InvocationTargetException e2) {}
		catch (InstantiationException e3) {}

		return null;*/
	}

	/** Open a temporary TCP connection to determine the outgoing IP for
	 * communication with the target IP
	 * @return Determined IP address
	 */
	public static IPaddr determineIP (String host, int port)
	{
		try
		{
			Socket sock = new Socket(host, port);
			IPaddr ret = new IPaddr(sock.getLocalAddress());
			Env.log.msg(Logger.DBG, ""+ret);
			sock.close();
			return ret;
		}
		catch (IOException e)
		{
			return null;
		}
	}
}
