package CTL;

import CTL.Comm.*;
import CTL.Types.*;
import NCLib.*;

import java.util.*;

/** Global environment */
public class Env
{
	/** Map with all allocated objects and their IDs */
	public static ObjectMap map;
	/** Logging handler */
	public static Logger log;
	/** Remote debugger */
	public static PeerID debugger = null;
	/** Map of custom AnyObj identifier strings */
	public static AnyObjIdentMap AnyObjIdent;
	/** Map of dynamic typedefs, read from './classdef.txt' */
	public static AnyObjIdentMap TypeDefs;
	/** List of running CTL.Processes */
	public static LinkedList<CTL.Process> procs;
	/** List of available Environments */
	private static LinkedList<Env> envs;
	/** JVM's listening port for debuggers */
	public static int debugID = -1;
	
	public static boolean clean = false;
	public static boolean daemon = false;
	public static boolean isDebugger = false;

	/** Process group */
	public Group grp;
	/** Communicator */
	public Communicator[] comm;

	static
	{
		log = null;
		map = new ObjectMap();
		AnyObjIdent = new AnyObjIdentMap();
		procs = new LinkedList<CTL.Process>();
		envs = new LinkedList<Env>();
		TypeDefs = new AnyObjIdentMap();

		// Init the Any map
		AnyObjIdent.put("bool", Boolean.TYPE);
		AnyObjIdent.put("char", Character.TYPE);
		// int1 not available
		AnyObjIdent.put("int2", Short.TYPE);
		AnyObjIdent.put("int4", Integer.TYPE);
		AnyObjIdent.put("int4", Integer.class);
		AnyObjIdent.put("int8", Long.TYPE);
		AnyObjIdent.put("real4", Float.TYPE);
		AnyObjIdent.put("real8", Double.TYPE);
		AnyObjIdent.put("void", Void.TYPE);
		// FIXME: hack hack hack
		AnyObjIdent.put("ctl::exception", String.class);

		// Init the typedefs
		for (String eq : Files.readAll("classdef.txt").split("\n"))
		{
			if (eq.startsWith("#"))
				continue;
			String[] bar = eq.split("=");
			if (bar.length < 2)
				continue;
			try
			{
				TypeDefs.put(bar[0], bar[1]);
			}
			catch (ClassNotFoundException e)
			{
				log.msg(Logger.ERR, "Mapping "+bar[0]+"->"+bar[1]+" could not be added.");
			}
		}
	}

	private Env ()
	{
		comm = null;
		grp = null;
	}

	public static Env newEnv ()
	{
		Env env = new Env();
		envs.add(env);
		return env;
	}

	public static Logger newLogger (String fname, boolean mangle)
	{
		if (log == null)
			log = new Logger(fname, Globals.LogLevel, mangle);
		return log;
	}

	public static Logger newLogger (String fname)
	{
		return newLogger(fname, true);
	}
}
