package CTL;

import CTL.Serialize.*;
import CTL.Types.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** Handle reading and writing of Java standard library types.
 *	Types currently handled:
 *		* HashMap
 *		* HashSet
 *		* LinkedList
 *		* Stack
 *		* TypeTree (CTL specific type)	
 *		* Vector
 */
public class Java
{
	public static <T extends Object> void writeLinkedList (SerialOut out, 
		LinkedList<T> list) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.writeInt(list.size());
		for (T o : list)
			out.serialWrite(o);
	}

	public static <T extends Object> LinkedList<T> readLinkedList (SerialIn in, 
		TypeTree targs) throws IOException, IllegalAccessException,
		InvocationTargetException, ClassNotFoundException, 
		InstantiationException
	{
		LinkedList<T> list = new LinkedList<T>();
		int sz = in.readInt();
		for (int i=0;i<sz;i++)
		{
			T moo = (T)in.serialRead(targs);
			list.add(moo);
		}
		return list;
	}

	public static void writeTypeTree (SerialOut out, TypeTree tree) throws
		IOException, IllegalAccessException, InvocationTargetException, 
		CTLException
	{
		out.serialWrite(tree.klasse().getName());
		out.serialWrite(tree.targs());
	}

	public static TypeTree readTypeTree (SerialIn in) throws IOException, 
		IllegalAccessException, InvocationTargetException, 
		ClassNotFoundException, InstantiationException
	{
		Class klasse = Class.forName(in.readString());
		LinkedList<TypeTree> targs = (LinkedList<TypeTree>)
			in.serialRead(LinkedList.class, TypeTree.class);
		return new TypeTree(klasse, targs);
	}

	public static <K,V extends Object> void writeHashMap (SerialOut out, 
		HashMap<K,V> map) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.writeInt(map.size());
		for (K key : map.keySet())
		{
			out.serialWrite(key);
			out.serialWrite(map.get(key));
		}
	}
	
	public static <K,V extends Object> HashMap<K,V> readHashMap (SerialIn in, 
		TypeTree targs) throws IOException, IllegalAccessException,
		InvocationTargetException, ClassNotFoundException, 
		InstantiationException
	{
		HashMap<K,V> map = new HashMap<K,V>();
		int sz = in.readInt();
		for (int i=0;i<sz;i++)
		{
			// TODO: Split template arguments somehow
			K moo = (K)in.serialRead(targs);
			V bar = (V)in.serialRead(targs);
			map.put(moo, bar);
		}
		return map;
	}
	
	public static <E extends Object> void writeHashSet (SerialOut out, 
		HashSet<E> set) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.writeInt(set.size());
		for (E entry : set)
			out.serialWrite(entry);
	}
	
	public static <E extends Object> HashSet<E> readHashSet (SerialIn in, 
		TypeTree targs) throws IOException, IllegalAccessException,
		InvocationTargetException, ClassNotFoundException, 
		InstantiationException
	{
		HashSet<E> set = new HashSet<E>();
		int sz = in.readInt();
		for (int i=0;i<sz;i++)
		{
			E entry = (E)in.serialRead(targs);
			set.add(entry);
		}
		return set;
	}
	
	public static <E extends Object> void writeVector (SerialOut out, 
		Vector<E> v) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.writeInt(v.size());
		for (E entry : v)
			out.serialWrite(entry);
	}
	
	public static <E extends Object> Vector<E> readVector (SerialIn in, 
		TypeTree targs) throws IOException, IllegalAccessException,
		InvocationTargetException, ClassNotFoundException, 
		InstantiationException
	{
		Vector<E> v = new Vector<E>();
		int sz = in.readInt();
		for (int i=0;i<sz;i++)
		{
			E entry = (E)in.serialRead(targs);
			v.add(entry);
		}
		return v;
	}

	public static <E extends Object> void writeStack (SerialOut out, Stack<E>
		stack) throws IOException, IllegalAccessException, 
		InvocationTargetException, CTLException
	{
		out.writeInt(stack.size());
		Stack<E> cloneOfMyOwn = (Stack<E>)stack.clone();
		for (int i=0;i<stack.size();i++)
			out.serialWrite(cloneOfMyOwn.pop());
	}

	public static <E extends Object> Stack<E> readStack (SerialIn in, 
		TypeTree targs) throws IOException, IllegalAccessException,
		InvocationTargetException, ClassNotFoundException,
		InstantiationException
	{
		Stack<E> stack = new Stack<E>();
		int sz = in.readInt();
		for (int i=0;i<sz;i++)
		{
			E entry = (E)in.serialRead(targs);
			stack.push(entry);
		}
		return stack;
	}
}
