package CTL.Serialize;

import CTL.*;
import CTL.Streams.DataOutputStream2;
import CTL.Types.*;
import NCLib.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

public abstract class SerialOut extends DataOutputStream2
{
	public SerialOut (OutputStream out) throws IOException
	{
		super(out);
	}
	
	public abstract void writeString (String data) throws IOException;
	public abstract void writeWString (String data) throws IOException;

	public abstract <T extends Object> void writeArray (T data) throws 
		IOException, IllegalAccessException, InvocationTargetException;
	public abstract <T extends Object> void __writeArray (T data) throws 
		IOException, IllegalAccessException, InvocationTargetException;
	public abstract void writeObject (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException;
	protected abstract void writeData (Object data) throws IOException,
		IllegalAccessException, InvocationTargetException, CTLException;

	/** Write an object to the stream (if one of the above function applies
	 * better to the data given, the relevant function will be called 
	 * automagically.
	 * @param data Value
	 */
	public void serialWrite (Object data) throws IOException, 
		IllegalAccessException, InvocationTargetException, CTLException
	{
		if (data==null)
		{
			if (Globals.NullExcept)
				throw new CTLException("No data to write");
			else
			{
				Env.log.msg(Logger.WARN, "No data to write.");
				return;
			}
		}
		
		Env.log.msg(Logger.DBG, "serialWrite(): "+
			data.getClass().getName());
		
		if (data instanceof String)
		{
			writeString((String)data);
			return;
		}
		
		if (data.getClass().isArray())
		{
			writeArray(data);
			return;
		}

		if (data instanceof Boolean)
		{
			writeBoolean((Boolean)data);
			return;
		}

		if (data instanceof Byte)
		{
			writeByte((Byte)data);
			return;
		}

		if (data instanceof Character)
		{
			writeChar((Character)data);
			return;
		}

		if (data instanceof Double)
		{
			writeDouble((Double)data);
			return;
		}

		if (data instanceof Float)
		{
			writeFloat((Float)data);
			return;
		}

		if (data instanceof Integer)
		{
			writeInt((Integer)data);
			return;
		}

		if (data instanceof Long)
		{
			writeLong((Long)data);
			return;
		}

		if (data instanceof Short)
		{
			writeShort((Short)data);
			return;
		}


		// TODO: This should all use the abstract interfaces to be more
		// applicable...

		Class[] ifs = data.getClass().getInterfaces();
		boolean foo = Classes.isIn(ifs, java.util.List.class);
		ClassInfo cinfo2 = new ClassInfo(data.getClass());
		if (cinfo2.implementing("java.util.List"))
		{
			List l = (List)data;
			writeInt(l.size());
			for (Object o : l)
				serialWrite(o);
			return;
		}

		// Handle Java standard library classes
		if (data instanceof LinkedList)
		{
			Java.writeLinkedList(this, (LinkedList)data);
			return;
		}

		if (data instanceof HashMap)
		{
			Java.writeHashMap(this, (HashMap)data);
			return;
		}

		if (data instanceof HashSet)
		{
			Java.writeHashSet(this, (HashSet)data);
			return;
		}

		if (data instanceof Stack)
		{
			Java.writeStack(this, (Stack)data);
			return;
		}

		if (data instanceof Vector)
		{
			Java.writeVector(this, (Vector)data);
			return;
		}

		if (data instanceof TypeTree)
		{
			Java.writeTypeTree(this, (TypeTree)data);
			return;
		}

		/*Env.log.msg(Logger.DBG, "hier bin ich.");
		System.out.println(data.getClass());
		Env.log.msg(Logger.DBG, "---+++---moo1");*/
		ClassInfo cinfo = new ClassInfo(data.getClass());
		//Env.log.msg(Logger.DBG, "---+++---moo3");
		if (!cinfo.implementing("CTL.Serialize.Writable"))
		{
			Env.log.msg(Logger.ERR, "OOStream: Class "+cinfo.fqcn()+
				" does not implement the Writable-Interface.");
			return;
		}
		
		writeData(data);
	}
}
