package CodeGen;

import CTL.*;
import ReflWrap.*;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/** Code generator for Java remote interfaces */
public class GenRI extends Gen
{
	public GenRI (String klass, HashMap<String,String> replaceMe, String targetDir) 
		throws ClassNotFoundException
	{
		super(klass, replaceMe, targetDir);
		name = classNamer(cinfo, "RI");
		if (isCC())
			supername = "RI";
		else
			supername = classNamer(cinfo, "CI");
	}
	
	/** Generate constructor wrappers
	 * @return Code
	 */
	protected String genConstructors () throws IOException
	{
		StringBuffer ret = new StringBuffer();

		if (constr.length == 0)
			ret.append("public "+name+ " ()\n{\nsuper((Object)null);\n}\n");

		for (int i=0;i<constr.length;i++)
		{
			Konstruktor con = constr[i];

			if (isCTLI)
			{
				ret.append("public "+name+" ("+con.params()+"\n");
            	ret.append("{\nthis(proc()"+((con.len()==0) ? "" : ", "));
            	String arglist = "";
            	for (int j=0;j<con.len();j++)
                	arglist += "arg"+j+((j==con.len()-1) ? "" : ", ");
            	ret.append(arglist);
            	ret.append(");\n}\n");
			}
			
			ret.append("public "+name+" (CTL.Process proc"+
				((con.len()==0) ? "" : ", ")+con.params()+"\n");
			ret.append("{\n");
			if (!supername.equals("RI"))
				ret.append("super((Object)null);\n");
			if (cinfo.isAnot(CTL.Annotate.Library.class))
			{
				ret.append("}\n");
				continue;
			}
			ret.append("Env env = proc.getEnv();\n");
			ret.append("try\n{\n");
			ret.append("FID fid = new FID((short)"+i+", \""+prfx+name+":"+i+"C\");\n");
			ret.append("Class moongoo = Class.forName(\"ReflWrap.TypeTree\");\n");
			ret.append("Constructor c = rResult.class.getConstructor(moongoo);\n");
			ret.append("Object[] objs = new Object[1];\n");
			ret.append("objs[0] = (Object)new TypeTree(rPointer.class);\n");
			ret.append("long objID = env.map.createObj(c, objs);\n");
			ret.append("IStream2 args = new IStream2();\n");
			
			for (int j=0;j<con.len();j++)
			{
				ret.append("args.write(arg"+j+");\n");
			}
			
			ret.append("Communicator comm = env.comm[proc().ID()];\n");
			ret.append("Header head = new Header(0, Remote.RMI, env.grp.myInfo());\n");
			ret.append("rPointer rp = new rPointer(objID, env.grp.myInfo());\n");
			ret.append("Remote.call(comm, head, 0, fid, args, rp);\n");
			ret.append("rResult foo = (rResult)env.map.getObj((int)objID);\n");
			ret.append("if (foo==null)\n{\n");
			ret.append("env.log.msg(Logger.ERR, \"No data was "+
				"received.\");\n");
			ret.append("throw new RuntimeException(\"No data was received."+
				"\");\n}\n");
			ret.append("while (!foo.received(foo.results()-1));\n");
			ret.append("rPointer tmp = (rPointer)foo.ObjectValue(foo.results()-1);\n");
			ret.append("setObjID(tmp.objID());\n");
			ret.append("setPeerID(tmp.peerID());\n");
			ret.append("env.log.msg(Logger.ERR, \"Object created with "+
				"ObjectID \"+objID()+\".\");\n");
			ret.append("}\ncatch (Exception e)\n{\n");
			ret.append("env.log.msg(Logger.ERR, \"Constructor "+
				"wrapper:\");\n");
			ret.append("RUtil.except(e);\n}\n}\n");
		}

		return ret.toString();
	}

	// TODO: Cleanup generated code for void methods
	/** Generate function wrappers
	 * @return Code
	 */
	protected String genWrappers () throws IOException, NoSuchMethodException
	{
		int me=1, s=1;
		StringBuffer ret = new StringBuffer();

		for (int i=0;i<methods.length;i++)
		{
			Methode meth = methods[i];
			// If there are unassigned FIDs, skip them.
			if ((meth == null) || (!meth.here()))
				continue;
			
			if (meth.dont())
			{
				if (meth.isStatic())
					me++;
				else
					s++;
				continue;
			}

			ClassInfo retVal = meth.minfo().returns();

			// Pretty hackish implementation of templated return types
			if (retVal == null)
			{
				String r = meth.minfo().m().getGenericReturnType().toString();

				if (replaceMe.size() > 0)
				{
					for (String key : replaceMe.keySet())
					{
						if (key.equals(r))
						{
							String rep = replaceMe.get(key);
							try
							{
								retVal = new ClassInfo(rep);
							}
							catch (ClassNotFoundException e)
							{
								Class c = Refl.getClass(rep);
								if (c == null)
								{
									System.out.println(rep);
									retVal = null;
								}
								else
									retVal = new ClassInfo(c);
							}
						}
					}
				}

				if (retVal == null)
					throw new RuntimeException("Source class contains undefined types.");

				meth.setReturnValue(retVal);
				meth.minfo().setReturnValue(retVal);
			}

			//System.out.println(retVal);

			// Two wrappers are created:
			//	1. Simple wrapper
			// 	2. rResult wrapper
			for (int rres=0;rres<2;rres++)
			{
				boolean rr = rres==1;

				if (meth.isVoid() && rr)
					continue;
				ret.append(meth.head(rr));
				
				// Static methods can be called with an optional CTL.Process argument
				if (meth.isStatic())
				{
					ret.append("\n{\n"+
						(meth.isVoid() ? "" : "return ")+
						meth.name()+(rr ? "_rr" : "")+"(proc()"+
						((meth.len()==0) ? "" : ", "));
					for (int j=0;j<meth.len();j++)
						ret.append("arg"+j+((j==meth.len()-1) ? "" : ", "));
					ret.append(");\n}\n"+meth.modifiers()+
						(rr ? "rResult" : retVal)+
						" "+meth.name()+(rr ? "_rr" : "")+
						" (CTL.Process proc"+((meth.len()==0) ? "" : ", ")+
						meth.params()+"\n");
				}
			
				ret.append("\n{\n");

				// Type of the return value is either the actual return type or a rResult.
				if (!meth.isVoid())
					if (rr)
						ret.append("rResult res = null;\n");
					else
						ret.append(meth.returnsT()+" res = "+
							Refl.defaultVal(meth.returns())+";\n");
				
				if (meth.isStatic())
					ret.append("try\n{\n"+"if (proc==null)\n{\n"+
						"Env.log.msg(Logger.ERR, "+
							"\"No remote process locator defined.\");\n"+
						"throw new RuntimeException(\"No remote process "+
						"locator defined.\");\n}\n");
				else
					ret.append("CTL.Process proc = proc();\n");

				ret.append("Env env = proc.getEnv();\n");

				if (!meth.isStatic())
					ret.append("try\n{\n"+"if (objID()==-1)\n"+
						"throw new Exception(\"ObjectID not set!\");\n");
			
				ret.append("Class moongoo = (new TypeTree[0]).getClass();\n");
				ret.append("Constructor c = "+
						"rResult.class.getConstructor(moongoo);\n");
				
				// Code for the rResult construction
				int nret=0;
				for (int k=0;k<meth.minfo().getParams().length;k++)
					if (!Refl.hasAnot("@CTL.Annotate.const_()", 
								meth.minfo().getParamAnot(k)) && !meth._const())
						nret++;
				ret.append("TypeTree[] argz = new TypeTree["+(nret+1)+"];\n");
				int p=0;
				for (int k=0;k<meth.minfo().getParams().length;k++)
					if (!Refl.hasAnot("@CTL.Annotate.const_()", 
							meth.minfo().getParamAnot(k)) && !meth._const())
					{
						String arg = Refl.class_(meth.minfo().getParams()[k]);
						if (ByteCode.matches(meth.minfo().params()[k], ".*<.*"))
							arg = "\""+meth.minfo().params()[k]+"\"";
						ret.append("argz["+(p++)+"] = new TypeTree("+
							arg+");\n");
					}

				String arg = Refl.class_(retVal._class());
				if (ByteCode.matches(meth.returnsT(), ".*<.*"))
					arg = "\""+meth.returnsT()+"\"";
				ret.append("argz["+nret+"] = new TypeTree("+arg+");\n"+
					"Object[] objs = new Object[1];\n"+
					"objs[0] = (Object)argz;\n"+
					"long objID = env.map.createObj(c, objs);\n"+
					"IStream2 args = new IStream2();\n");
			
				for (int j=0;j<meth.len();j++)
					ret.append("args.write(arg"+j+");\n");

				ret.append("Communicator comm = env.comm[proc().ID()];\n");
				ret.append("FID fid = new FID((short)");
				
				if (meth.isStatic())
				{
					String stat = "S";
					if (cinfo.isAnot(CTL.Annotate.Library.class))
					{
						// TODO: hack
						stat = Refl.java2cpp(retVal._class());
						String[] ppp = meth.minfo().params();
						for (int iii=0;iii<ppp.length;iii++)
						{
							Class c = null;
							String tt = "null";

							try
							{
								c = Class.forName(ppp[iii]);
								tt = Refl.java2cpp(c);
							}
							catch (ClassNotFoundException e)
							{
								//System.out.println(ppp[iii]);
							}

							if (tt.equals("null") && (replaceMe.size() > 0))
							{
								for (String key : replaceMe.keySet())
								{
									if (key.equals(ppp[iii]))
									{
										String rep = replaceMe.get(key);
										//System.out.println(rep);
										try
										{
											c = Class.forName(rep);
											tt = Refl.java2cpp(c);
										}
										catch (ClassNotFoundException e)
										{
											c = Refl.getClass(rep);
											tt = Refl.java2cpp(c);
										}
									}
								}
							}
							if (tt.equals("null"))
								throw new RuntimeException(
									"Source class contains undefined types.");
							stat += ","+tt;
						}
						//System.out.println(stat);
					}
					ret.append(me+", \""+
						prfx+name+":"+me+stat+"\");\n"+
						"Header head = new Header(0, Remote.RMI, env.grp.myInfo());\n"+
						"rPointer rp = new rPointer(objID, env.grp.myInfo());\n"+
						"Remote.call(comm, head, 0, fid, args, rp);\n\n");
					me+=rres;
				}
				else
				{
					ret.append(s+", \""+"\");\n"+
						"Header head = new Header(0, Remote.RMI, env.grp.myInfo());\n"+
						"rPointer rp = new rPointer(objID, env.grp.myInfo());\n"+
						"Remote.call(comm, head, objID(), fid, args, rp);\n\n");
					s+=rres;
				}

				if (!meth.isVoid())
				if (rr)
					ret.append("res = (rResult)env.map.getObj((int)objID);\n");
				else
				{
					ret.append("rResult foo = (rResult)env.map.getObj((int)"+
						"objID);\n"+
						"while (!foo.received(foo.results()-1));\n");

					String[] val = genValue(Refl.rtype(retVal._class()));
					Class[] args = new Class[1];
					args[0] = Integer.TYPE;
					Method m = rResult.class.getDeclaredMethod(
						Refl.lastToken(val[0], "."), args);
					ret.append("res = ");
					if (retVal.isArray())
						ret.append(convertArray(meth.minfo())+"(");
					ret.append(val[1]+"foo."+m.getName()+"(foo.results()-1)");
					if (retVal.isArray())
						ret.append(")");
					ret.append(";\n");
					ret.append("env.log.msg(Logger.DBG2, \"rResult: \""+
						"+(foo.results()-1)+\" used.\");\n");

					for (int k=0;k<meth.minfo().getParams().length;k++)
						if (!Refl.hasAnot("@CTL.Annotate.const_()", 
									meth.minfo().getParamAnot(k)) && !meth._const())
						{
							String[] val2 = genValue(Refl.rtype(
								meth.minfo().getParams()[k]));
							Class[] args2 = new Class[1];
							args2[0] = Integer.TYPE;
							Method muh = rResult.class.getDeclaredMethod(
								val2[0], args2);
							ret.append("env.log.msg(Logger.DBG, "+
								"\"non-const arg "+k+" is \"+foo."+
								muh.getName()+"("+k+"));\n");
						}
				}
				
				ret.append("}\ncatch (Exception e)\n{\n"+
					"Env.log.msg(Logger.ERR, \""+
					(rr ? "rResult w" : "W")+"rapper:\");\n"+
					"RUtil.except(e);\n}\n");
		
				if (!meth.isVoid())
					ret.append("return res;\n");
				
				ret.append("}\n");
			}
		}

		return ret.toString();
	}
	
	protected String genAccept () throws IOException
	{
		if (isCTLI)
		{
			//return GenCI.genAccept(this);
			String ret = "private static CTL.Process proc2 = null;\n";
			ret += genGetProc();
			return ret;
		}
		return super.genAccept();
	}
}
